"use strict";

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _fs = require("fs");
var _os = require("os");
var _path = require("path");

var async = require("neo-async");
var rimraf = require("rimraf");
var webpack = require("webpack");
var workerFarm = require("worker-farm");

var workers = workerFarm(require.resolve("./build_worker"));

var webpackConfig = require("../config/build");
var write = require("./write");

module.exports = function (config) {
  return new Promise(function (resolve, reject) {
    var output = config.antwar.output;

    if (!output) {
      return reject(new Error("Missing output directory"));
    }

    return webpackConfig(config).then(runWebpack()).then(generateParameters(config.antwar, config.webpack)).then(writePages(config.antwar)).then(removeSiteBundle(output)).then(resolve).catch(reject);
  });
};

function runWebpack() {
  return function (config) {
    return new Promise(function (resolve, reject) {
      webpack(config, function (err, stats) {
        if (err) {
          return reject(err);
        }

        if (stats.hasErrors()) {
          return reject(stats.toString("errors-only"));
        }

        return resolve(stats);
      });
    });
  };
}

function generateParameters(antwarConfig, webpackConfig) {
  var publicPath = webpackConfig.output ? webpackConfig.output.publicPath : "";

  return function (stats) {
    return new Promise(function (resolve) {
      var assets = stats.compilation.assets;
      var cssFiles = Object.keys(assets).map(function (asset) {
        if (_path.extname(asset) === ".css") {
          return assets[asset].existsAt;
        }

        return null;
      }).filter(function (a) {
        return a;
      });
      var jsFiles = [];

      // Copy template configuration to webpack side so HtmlWebpackPlugin picks it up
      var template = _extends({
        cssFiles: [],
        jsFiles: []
      }, antwarConfig.template);

      var cwd = process.cwd();
      var site = require(_path.join(cwd, antwarConfig.output, "site.js"));
      var parameters = {
        cwd: cwd,
        renderPage: site.renderPage,
        allPages: site.allPages,
        output: _path.join(cwd, antwarConfig.output),
        config: antwarConfig,
        cssFiles: cssFiles,
        jsFiles: jsFiles,
        templates: {
          page: _extends({}, template, {
            // XXX: sync operation
            file: _fs.readFileSync(template && template.file || _path.join(__dirname, "../../templates/page.ejs"), {
              encoding: "utf8"
            }),
            cssFiles: cssFiles.map(function (cssFile) {
              return publicPath + "/" + _path.basename(cssFile);
            }),
            jsFiles: jsFiles
          }),
          // TODO: expose to the user?
          interactive: {
            file: _fs.readFileSync(_path.join(__dirname, "../../templates/interactive.ejs"), {
              encoding: "utf8"
            })
          },
          interactiveIndex: {
            file: _fs.readFileSync(_path.join(__dirname, "../../templates/interactive_index.ejs"), {
              encoding: "utf8"
            })
          }
        }
      };

      resolve(parameters);
    });
  };
}

function writePages(antwarConfig) {
  return function (parameters) {
    return new Promise(function (resolve, reject) {
      var config = parameters.config;
      var assets = config && config.assets ? config.assets : [];

      if (parameters.cssFiles) {
        parameters.cssFiles.forEach(function (cssFile) {
          assets.push({
            from: cssFile,
            to: "./" + _path.basename(cssFile)
          });
        });
      }

      write.pages(parameters)(function (err, tasks) {
        if (err) {
          return reject(err);
        }

        executeTasks(tasks, antwarConfig.maximumWorkers, antwarConfig.console.log).then(function () {
          return resolve(parameters);
        }).catch(reject);
      });
    });
  };
}

function executeTasks(tasks, maximumWorkers, log) {
  return new Promise(function (resolve, reject) {
    async.eachLimit(tasks, maximumWorkers || _os.cpus().length, function (o, cb) {
      log("Starting to write pages");

      workers(o, function (err) {
        log("Finished writing pages");

        cb(err);
      });
    }, function (err) {
      log("BUILD FINISHED!");

      workerFarm.end(workers);

      if (err) {
        return reject(err);
      }

      return resolve();
    });
  });
}

function removeSiteBundle(outputDirectory) {
  return function (parameters) {
    return new Promise(function (resolve, reject) {
      rimraf(_path.join(process.cwd(), outputDirectory, "site.js"), function (err) {
        if (err) {
          return reject(err);
        }

        return resolve(parameters);
      });
    });
  };
}