"use strict";

var _ = require("lodash");
var parseSectionPages = require("./parse-section-pages");
var transformSections = require("./transform-sections");

describe("Transform sections", function () {
  it("transforms a root section", function () {
    var sectionName = "/";
    var section = {
      layout: function layout() {},
      transform: function transform(pages) {
        return _.sortBy(pages, function (page) {
          return page.file.sort;
        });
      }
    };
    var parsedPages = parseSectionPages(sectionName, section, context(function (path) {
      return ["./" + path];
    }));
    var result = transformSections(sectionName, section, parsedPages);
    var expected = [{
      type: "page",
      fileName: "first.md",
      file: {
        sort: 0
      },
      layout: undefined,
      section: section,
      sectionName: "/",
      url: "/first/"
    }, {
      type: "page",
      fileName: "second.md",
      file: {
        sort: 1
      },
      layout: undefined,
      section: section,
      sectionName: "/",
      url: "/second/"
    }, {
      type: "page",
      fileName: "third.md",
      file: {
        sort: 10
      },
      layout: undefined,
      section: section,
      sectionName: "/",
      url: "/third/"
    }, {
      type: "index",
      fileName: "index.md",
      file: {
        sort: -1
      },
      layout: undefined,
      section: section,
      sectionName: "/",
      url: "/"
    }];

    expect(result).toEqual(expected);
  });

  it("transforms a nested page", function () {
    var sectionName = "/";
    var section = {
      layout: function layout() {},
      transform: function transform(pages) {
        return _.sortBy(pages, function (page) {
          return page.file.sort;
        });
      }
    };
    var parsedPages = parseSectionPages(sectionName, section, context(function (path) {
      return ["./" + path];
    }, function (file) {
      return "nested/page/" + file;
    }));
    var result = transformSections(sectionName, section, parsedPages);
    var expected = [{
      type: "page",
      fileName: "nested/page/first.md",
      file: {
        sort: 0
      },
      layout: undefined,
      section: section,
      sectionName: "/",
      url: "/nested/page/first/"
    }, {
      type: "page",
      fileName: "nested/page/second.md",
      file: {
        sort: 1
      },
      layout: undefined,
      section: section,
      sectionName: "/",
      url: "/nested/page/second/"
    }, {
      type: "page",
      fileName: "nested/page/third.md",
      file: {
        sort: 10
      },
      layout: undefined,
      section: section,
      sectionName: "/",
      url: "/nested/page/third/"
    }, {
      type: "index",
      fileName: "nested/page/index.md",
      file: {
        sort: -1
      },
      layout: undefined,
      section: section,
      sectionName: "/",
      url: "/nested/page/"
    }];

    expect(result).toEqual(expected);
  });

  it("transforms a child section", function () {
    var sectionName = "/";
    var section = {
      paths: {
        docs: {
          layout: function layout() {},
          transform: function transform(pages) {
            return _.sortBy(pages, function (page) {
              return page.file.sort;
            });
          }
        }
      }
    };
    var parsedPages = parseSectionPages(sectionName, section, context(function (path) {
      return ["./docs/" + path];
    }));
    var result = transformSections(sectionName, section, parsedPages);
    var expected = [{
      type: "page",
      fileName: "docs/first.md",
      file: {
        sort: 0
      },
      layout: undefined,
      section: section,
      sectionName: "docs",
      url: "/docs/first/"
    }, {
      type: "page",
      fileName: "docs/second.md",
      file: {
        sort: 1
      },
      layout: undefined,
      section: section,
      sectionName: "docs",
      url: "/docs/second/"
    }, {
      type: "page",
      fileName: "docs/third.md",
      file: {
        sort: 10
      },
      layout: undefined,
      section: section,
      sectionName: "docs",
      url: "/docs/third/"
    }, {
      type: "index",
      fileName: "docs/index.md",
      file: {
        sort: -1
      },
      layout: undefined,
      section: section,
      sectionName: "docs",
      url: "/docs/"
    }];

    expect(result).toEqual(expected);
  });

  it("transforms root and child section", function () {
    var sectionName = "/";
    var section = {
      layout: function layout() {},
      transform: function transform(pages) {
        return _.sortBy(pages, function (page) {
          return page.file.sort;
        });
      },
      paths: {
        docs: {
          layout: function layout() {},
          transform: function transform(pages) {
            return _.sortBy(pages, function (page) {
              return page.file.sort;
            });
          }
        }
      }
    };
    var parsedPages = parseSectionPages(sectionName, section, context(function (path) {
      return ["./" + path, "./docs/" + path];
    }));
    var result = transformSections(sectionName, section, parsedPages);
    var expected = [{
      type: "page",
      fileName: "first.md",
      file: {
        sort: 0
      },
      layout: undefined,
      section: section,
      sectionName: "/",
      url: "/first/"
    }, {
      type: "page",
      fileName: "second.md",
      file: {
        sort: 1
      },
      layout: undefined,
      section: section,
      sectionName: "/",
      url: "/second/"
    }, {
      type: "page",
      fileName: "third.md",
      file: {
        sort: 10
      },
      layout: undefined,
      section: section,
      sectionName: "/",
      url: "/third/"
    }, {
      type: "index",
      fileName: "index.md",
      file: {
        sort: -1
      },
      layout: undefined,
      section: section,
      sectionName: "/",
      url: "/"
    }, {
      type: "page",
      fileName: "docs/first.md",
      file: {
        sort: 0
      },
      layout: undefined,
      section: section,
      sectionName: "docs",
      url: "/docs/first/"
    }, {
      type: "page",
      fileName: "docs/second.md",
      file: {
        sort: 1
      },
      layout: undefined,
      section: section,
      sectionName: "docs",
      url: "/docs/second/"
    }, {
      type: "page",
      fileName: "docs/third.md",
      file: {
        sort: 10
      },
      layout: undefined,
      section: section,
      sectionName: "docs",
      url: "/docs/third/"
    }, {
      type: "index",
      fileName: "docs/index.md",
      file: {
        sort: -1
      },
      layout: undefined,
      section: section,
      sectionName: "docs",
      url: "/docs/"
    }];

    expect(result).toEqual(expected);
  });
});

function context(shapePath) {
  var shapeFile = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : id;

  var files = [{
    file: "index.md",
    sort: -1
  }, {
    file: "first.md",
    sort: 0
  }, {
    file: "third.md",
    sort: 10
  }, {
    file: "second.md",
    sort: 1
  }];

  var modules = _.fromPairs(_.flatMap(files.map(function (_ref) {
    var file = _ref.file,
        sort = _ref.sort;
    return {
      file: shapeFile(file),
      sort: sort
    };
  }), function (_ref2) {
    var file = _ref2.file,
        sort = _ref2.sort;
    return shapePath(file).map(function (name) {
      return [name, { sort: sort }];
    });
  }));
  var ret = function ret(name) {
    return modules[name];
  };
  ret.keys = function () {
    return Object.keys(modules);
  };

  return ret;
}

function id(a) {
  return a;
}